'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TranslationProvider = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactBroadcast = require('react-broadcast');

var _reduce = require('lodash/reduce');

var _reduce2 = _interopRequireDefault(_reduce);

var _kebabCase = require('lodash/kebabCase');

var _kebabCase2 = _interopRequireDefault(_kebabCase);

var _isString = require('lodash/isString');

var _isString2 = _interopRequireDefault(_isString);

var _isNumber = require('lodash/isNumber');

var _isNumber2 = _interopRequireDefault(_isNumber);

var _intlMessageformat = require('intl-messageformat');

var _intlMessageformat2 = _interopRequireDefault(_intlMessageformat);

var _countryData = require('country-data');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var CHANNEL = 'language';
var DEFAULT_LANG = 'en_US';
var customDefaultLanguage = DEFAULT_LANG;

var TranslationProvider = exports.TranslationProvider = function TranslationProvider(_ref) {
  var language = _ref.language,
      _ref$defaultLanguage = _ref.defaultLanguage,
      defaultLanguage = _ref$defaultLanguage === undefined ? DEFAULT_LANG : _ref$defaultLanguage,
      children = _ref.children;

  customDefaultLanguage = defaultLanguage;
  return _react2.default.createElement(
    _reactBroadcast.Broadcast,
    { channel: CHANNEL, value: language || customDefaultLanguage },
    _react2.default.createElement(
      'div',
      null,
      children
    )
  );
};

var translated = function translated(_ref2) {
  var translations = _ref2.translations,
      _ref2$params = _ref2.params,
      params = _ref2$params === undefined ? {} : _ref2$params,
      _ref2$mapTranslations = _ref2.mapTranslationsToProps,
      mapTranslationsToProps = _ref2$mapTranslations === undefined ? function (o) {
    return o;
  } : _ref2$mapTranslations,
      _ref2$format = _ref2.format,
      format = _ref2$format === undefined ? {} : _ref2$format;

  var warmTranslations = preHeat(translations, format);
  return function (Component) {
    return function (props) {
      return _react2.default.createElement(
        _reactBroadcast.Subscriber,
        { channel: CHANNEL },
        function (language) {
          return _react2.default.createElement(Component, _extends({}, props, mapTranslationsToProps(translateWithDefaults({
            translations: warmTranslations,
            language: language || customDefaultLanguage,
            reducer: templateReducer(templateParamValues(props, params))
          }), props)));
        }
      );
    };
  };
};

exports.default = translated;


var moneyFormat = function moneyFormat(language) {
  return {
    number: {
      money: {
        style: 'currency',
        currency: getRegionCurrency(language.slice(-2)).code,
        minimumFractionDigits: 0
      }
    }
  };
};

var preHeat = function preHeat(translations, format) {
  return (0, _reduce2.default)(translations, function (acc, o, language) {
    acc[language] = (0, _reduce2.default)(o, function (acc, v, k) {
      acc[k] = new _intlMessageformat2.default(v, (0, _kebabCase2.default)(language), _extends({}, moneyFormat(language), format));
      return acc;
    }, {});
    return acc;
  }, {});
};

var templateReducer = function templateReducer(values) {
  return function (acc, v, k) {
    acc[k] = v.format(values);
    return acc;
  };
};

var paramClone = function paramClone(o) {
  return (0, _reduce2.default)(o, function (acc, v, k) {
    if ((0, _isString2.default)(v) || (0, _isNumber2.default)(v)) acc[k] = v;
    return acc;
  }, {});
};

var templateParamValues = function templateParamValues(props, params) {
  return (0, _reduce2.default)(params, function (acc, fn, k) {
    acc[k] = fn(props);
    return acc;
  }, paramClone(props));
};

var translateWithDefaults = function translateWithDefaults(_ref3) {
  var _ref3$translations = _ref3.translations,
      translations = _ref3$translations === undefined ? {} : _ref3$translations,
      language = _ref3.language,
      reducer = _ref3.reducer;
  return (0, _reduce2.default)(_extends({}, translations[customDefaultLanguage], translations[language]), reducer, {});
};

var getRegionCurrency = function getRegionCurrency(region) {
  return _countryData.currencies[_countryData.countries[region].currencies[0]];
};